## $Id: browser.py 17936 2024-10-04 22:26:13Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import httplib
import hashlib
import grok
from kofacustom.nigeria.interswitch.browser import (
    InterswitchPaymentRequestWebservicePageApplicant,
    InterswitchPaymentRequestWebservicePageStudent,
    InterswitchPaymentVerifyWebservicePageApplicant,
    InterswitchPaymentVerifyWebservicePageStudent,
    InterswitchPageStudent, InterswitchPageApplicant,
    module_activated,
    )
from kofacustom.nigeria.interswitch.webcheckoutbrowser import (
    webcheckout_module_activated,
    WebCheckoutPageStudent,
    WebCheckoutPageApplicant,
    WebCheckoutConfirmTransactionStudent,
    WebCheckoutConfirmTransactionApplicant,
    )
from kofacustom.edopoly.students.interfaces import ICustomStudentOnlinePayment
from kofacustom.edopoly.applicants.interfaces import ICustomApplicantOnlinePayment
from kofacustom.edopoly.interfaces import MessageFactory as _

PRODUCT_ID = '7571' # must be provided by Interswitch
SITE_NAME = 'edopoly-kofa.waeup.org'
PROVIDER_ACCT = '0773411069'
PROVIDER_BANK_ID = '31'
PROVIDER_ITEM_NAME = 'WAeAC'
INSTITUTION_NAME = 'EdoPoly'
CURRENCY = '566'
GATEWAY_AMT = 250.0
MAC = '672ABFB2F3755A7793DE545BDA00F44878DF59140B43A1E7DF16245D299F3108AFF65502F16D95256B3242B43CBC3F512A208EB1BA977080D65328800C49912C'

POST_ACTION = 'https://webpay.interswitchng.com/paydirect/pay'
#POST_ACTION = 'https://sandbox.interswitchng.com/webpay/pay'
HOST = 'webpay.interswitchng.com'
#HOST = 'sandbox.interswitchng.com'
URL = '/paydirect/api/v1/gettransaction.json'
#URL = '/webpay/api/v1/gettransaction.json'

httplib.HTTPSConnection.debuglevel = 0
HTTPS = True

SPECIAL_PAYMENT_PARAMS = {

    'conv_nd': ('101', 1000.0, '0031913976', '121'),
    'conv_hnd': ('101', 1000.0, '0031913976', '121'),
    }


class CustomInterswitchPageStudent(InterswitchPageStudent):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomStudentOnlinePayment)
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    product_id = PRODUCT_ID
    mac = MAC

    def update(self):
        if not module_activated(
            self.context.student.current_session, self.context):
            self.flash(_('Forbidden'), type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        error = self.init_update()
        if error:
            self.flash(error, type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        self.context.r_company = u'interswitch'
        student = self.student
        xmldict = self.xmldict
        # Provider data
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        provider_amt = 0.0
        if not self.context.p_item == 'Balance':
            if self.context.p_category == 'ict_entre':
                provider_amt = 3000.0
            if self.context.p_category == 'clearance':
                provider_amt = 1500.0
            # temporarily disabled
            #if self.context.p_category in ('transcript', 'certificate'):
            #    provider_amt = 2000.0
        # Institution data
        xmldict['institution_acct'] = '0068241848'
        xmldict['institution_bank_id'] = '121'
        if self.context.p_category == 'union':
            xmldict['institution_acct'] = '0066437412'
        if self.context.p_category == 'nhis':
            xmldict['institution_acct'] = '0075379404'
        if self.context.p_category == 'cyber':
            xmldict['institution_acct'] = '1220055070'
            xmldict['institution_bank_id'] = '117'
        if self.context.p_category == 'ispon':
            xmldict['institution_acct'] = '1220055104'
            xmldict['institution_bank_id'] = '117'
        xmldict['institution_amt'] = '0.0'
        self.pay_item_id = '101' # must be provided by Interswitch
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['institution_item_name'] = self.context.category
        xmldict['institution_name'] = INSTITUTION_NAME
        xmldict['institution_amt'] = 100 * (
            self.context.amount_auth - provider_amt - GATEWAY_AMT)
        # Interswitch amount is not part of the xml data
        if provider_amt == 0:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        else:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = GATEWAY_AMT
        self.amount_auth = int(100 * self.context.amount_auth)
        hashargs = (
            self.context.p_id +
            PRODUCT_ID +
            self.pay_item_id +
            str(int(self.amount_auth)) +
            self.site_redirect_url +
            self.mac)
        self.hashvalue = hashlib.sha512(hashargs).hexdigest()
        return

class CustomInterswitchPageApplicant(InterswitchPageApplicant):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomApplicantOnlinePayment)
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    pay_item_id = '101' # must be provided by Interswitch
    product_id = PRODUCT_ID
    mac = MAC

    def update(self):
        if not module_activated(
            self.context.__parent__.__parent__.year, self.context):
            self.flash(_('Forbidden'), type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        error = self.init_update()
        if error:
            self.flash(error, type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        self.context.r_company = u'interswitch'
        xmldict = {}
        provider_amt = 1000.0
        if self.context.p_category != 'application':
            provider_amt = 0.0
        xmldict['institution_acct'] = '0068241848'
        xmldict['institution_bank_id'] = '121'
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        xmldict['institution_amt'] = 100 * (
            self.context.amount_auth - provider_amt - GATEWAY_AMT)
        xmldict['institution_item_name'] = self.context.category
        xmldict['institution_name'] = INSTITUTION_NAME
        if self.context.p_category in SPECIAL_PAYMENT_PARAMS.keys():
            self.pay_item_id = SPECIAL_PAYMENT_PARAMS[self.context.p_category][0]
            xmldict['institution_acct'] = SPECIAL_PAYMENT_PARAMS[
                self.context.p_category][2]
            xmldict['institution_bank_id'] = SPECIAL_PAYMENT_PARAMS[
                self.context.p_category][3]
            provider_amt = SPECIAL_PAYMENT_PARAMS[self.context.p_category][1]
            xmldict['institution_amt'] = 100 * (
                self.context.amount_auth - provider_amt - GATEWAY_AMT)
            xmldict['provider_amt'] = 100 * provider_amt
        # Interswitch amount is not part of the xml data
        if provider_amt == 0:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        else:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict

        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = GATEWAY_AMT
        self.amount_auth = int(100 * self.context.amount_auth)
        hashargs = (
            self.context.p_id +
            PRODUCT_ID +
            self.pay_item_id +
            str(int(self.amount_auth)) +
            self.site_redirect_url +
            self.mac)
        self.hashvalue = hashlib.sha512(hashargs).hexdigest()
        return

class CustomInterswitchPaymentRequestWebservicePageStudent(
    InterswitchPaymentRequestWebservicePageStudent):
    """Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = MAC

class CustomInterswitchPaymentVerifyWebservicePageStudent(
    InterswitchPaymentVerifyWebservicePageStudent):
    """Payment verify view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = MAC

class CustomInterswitchPaymentRequestWebservicePageApplicant(
    InterswitchPaymentRequestWebservicePageApplicant):
    """Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = MAC

class CustomInterswitchPaymentVerifyWebservicePageApplicant(
    InterswitchPaymentVerifyWebservicePageApplicant):
    """Payment verify view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = MAC

# WebCheckout customizations

MERCHANT_CODE = 'MX104253'
PAY_ITEM_ID = 'Default_Payable_MX104253'

class CustomWebCheckoutPageStudent(WebCheckoutPageStudent):
    """ View which sends a POST request to the Interswitch
    WebCheckout payment gateway.
    """
    action = 'https://newwebpay.interswitchng.com/collections/w/pay'
    #action = 'https://newwebpay.qa.interswitchng.com/collections/w/pay'
    currency = '566'
    pay_item_id = PAY_ITEM_ID
    merchant_code = MERCHANT_CODE
    gateway_amt = 0.0
    product_id = PRODUCT_ID

class CustomWebCheckoutPageApplicant(WebCheckoutPageApplicant):
    """ View which sends a POST request to the Interswitch
    WebCheckout payment gateway.
    """

    action = 'https://newwebpay.interswitchng.com/collections/w/pay'
    currency = '566'
    pay_item_id = PAY_ITEM_ID
    merchant_code = MERCHANT_CODE

class CustomWebCheckoutConfirmTransactionStudent(WebCheckoutConfirmTransactionStudent):
    """ Request webservice view for the WebCheckout gateway
    """
    merchant_code = MERCHANT_CODE
    gateway_host = 'webpay.interswitchng.com'
    gateway_url = '/collections/api/v1/gettransaction.json'
    https = True

class CustomWebCheckoutConfirmTransactionApplicant(WebCheckoutConfirmTransactionApplicant):
    """ Request webservice view for the WebCheckout gateway
    """
    merchant_code = MERCHANT_CODE
    gateway_host = 'webpay.interswitchng.com'
    gateway_url = '/collections/api/v1/gettransaction.json'
    https = True


